import java.awt.Frame;
import java.awt.Graphics;
import java.awt.TextField;
import java.awt.event.MouseListener;
import java.awt.event.TextListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.halfdog.io.parport.LinuxParPort;
import org.halfdog.io.parport.ParPort;

public class PortTest extends WindowAdapter implements TextListener {
  private ParPort		parport;

  private Frame			frame;
  private java.awt.Panel	pan;
  private ButtonCanvas		buttons;
  private TextField		outvaltextfield;

  private Logger		logger;

  public static void main(String argv[]) {
    if(argv.length!=1) {
      System.err.println("Usage: PortTest [device]");
      System.exit(1);
    }
    new PortTest(argv[0]);
  }


  public PortTest(String device) {
    logger=Logger.getLogger("PortTest");

    try {
      parport=new LinuxParPort(device);
    } catch (IOException ioe) {
      logger.log(Level.SEVERE, "Can't open device "+device, ioe);
      System.exit(1);
    }



// init the gui
    frame=new Frame("PortTest "+device);
    frame.setSize(300, 200);
    frame.addWindowListener(this);

    pan=new java.awt.Panel();

    outvaltextfield=new TextField("00", 3);
    outvaltextfield.addTextListener(this);

    buttons=new ButtonCanvas();
    buttons.setSize(200, 200);

    pan.add(buttons);
    pan.add(outvaltextfield);

    frame.add(pan);
    frame.setVisible(true);

//    buttons.setLocation(0, 0);
//    outvaltextfield.setLocation(220, 10);
  }




  public void windowClosing(WindowEvent e) {
    System.exit(1);
  }

//***************************************************************************//
//									     //
//			TextListener implementation			     //
//									     //
//***************************************************************************//

  public void textValueChanged(java.awt.event.TextEvent e) {
    int i;
    
    i=-1;
    try {
      i=Integer.parseInt(outvaltextfield.getText(), 16);
    } catch (NumberFormatException nfe) {
      logger.log(Level.WARNING, "Unparseable string {}",
                 outvaltextfield.getText());
      i=-1;
    }
    if((i<0)||(i>0xFF)) return;

    buttons.setOutData(i);
  }



class ButtonCanvas extends java.awt.Canvas implements MouseListener, Runnable {

  private int			flags, status, data_in, data_out;
  private Graphics		graph;


  public ButtonCanvas() {
    addMouseListener(this);
    Thread thr=new Thread(this);
    thr.start();
  }


/**Draw the whole display.
*/
  public void paint(Graphics g) {
    int i, n;

    this.graph=g;
    g.setColor(java.awt.Color.black);
    g.drawString("Data 0                 Data 7",  10, 15);
    g.drawString("Strobe /Feed Init Sel Read", 10, 55);
    g.drawString("Err Sel PE Ackn Busy",   10, 95);
    g.drawString("Data 0                 Data 7",  10, 135);

    for(i=0; i<8; i++) {
      g.setColor(java.awt.Color.black);
      g.drawRect(10+i*20, 20, 16, 16);
      if((data_out&(1<<i))!=0) g.setColor(java.awt.Color.red);
      else g.setColor(java.awt.Color.gray);
      g.fillRect(11+i*20, 21, 15, 15);
    }

// flags
    n=flags;
    if((flags&parport.CTRL_DIRECTION)!=0) n|=0x10;
    for(i=0; i<5; i++) {
      g.setColor(java.awt.Color.black);
      g.drawRect(10+i*20, 60, 16, 16);
      if((n&(1<<i))!=0) g.setColor(java.awt.Color.red);
      else g.setColor(java.awt.Color.gray);
      g.fillRect(11+i*20, 61, 15, 15);
    }

// status byte
    n=status;
    for(i=0; i<5; i++) {
      g.setColor(java.awt.Color.black);
      g.drawRect(10+i*20, 100, 16, 16);
      if((status&(1<<(i+3)))!=0) g.setColor(java.awt.Color.red);
      else g.setColor(java.awt.Color.gray);
      g.fillRect(11+i*20, 101, 15, 15);
    }

// input byte
    for(i=0; i<8; i++) {
      g.setColor(java.awt.Color.black);
      g.drawRect(10+i*20, 140, 16, 16);
      if((data_in&(1<<i))!=0) g.setColor(java.awt.Color.red);
      else g.setColor(java.awt.Color.gray);
      g.fillRect(11+i*20, 141, 15, 15);
    }

  }



  public void setOutData(int val) {
    int i, n;

    if(graph==null) return;

    if(data_out!=val) {
      for(i=0; i<8; i++) {
        n=(1<<i);
        if((data_out&n)!=(val&n)) {
          if((val&n)==0) graph.setColor(java.awt.Color.gray);
          else graph.setColor(java.awt.Color.red);

          System.err.println("ButtonCanvas.139: redrawing field "+i+
                             " with value "+Integer.toHexString(val&n));
          graph.fillRect(11+i*20, 21, 15, 15);
        }
      }
      repaint();
    }

    data_out=val;
    System.err.println("ButtonCanvas.144: new value is "+
                       Integer.toHexString(val));

    String str=Integer.toHexString(val);
    if(!str.equals(outvaltextfield.getText())) outvaltextfield.setText(str);

    try {
      parport.write((byte)data_out);
    } catch (IOException ioe) {
      System.err.println("ButtonCanvas.250: unexpected ioexception");
      ioe.printStackTrace();
      System.exit(1);
    }
  }


/** Update the flags of the control lines.
 */
  void setFlags() {
    try {
      parport.writeControl((byte)(flags^parport.CTRL_INVERT));
      repaint();
    } catch (IOException ioe) {
      System.err.println("ButtonCanvas.199: unexpected exception occured.");
      ioe.printStackTrace();
      System.exit(1);
    }
  }


//***************************************************************************//
//									     //
//			MouseListener implementation			     //
//									     //
//***************************************************************************//

  public void mouseClicked(java.awt.event.MouseEvent e) {
    int i, n;

    System.err.println("PortText.mouseClicked("+e.getComponent().getName()+
                       ").122");

    i=e.getY();
    if((i>=20)&&(i<=35)) {
      i=e.getX()-10;
      if((i>159)||((i%20)>15)) return;

      i=i/20;
      System.err.println("PortTest.132: setting data bit "+i);
      n=data_out^(1<<i);
      setOutData(n);
      return;
    }
    if((i>=60)&&(i<=75)) {
      i=e.getX()-10;
      if((i>99)||((i%20)>15)) return;

      i=i/20;
      if(i==4) i++;		// direction bit move
      System.err.println("PortTest.221: setting ctl bit "+i);
      flags^=(1<<i);
      setFlags();
    }
  }

  public void mouseEntered(java.awt.event.MouseEvent e) {}
  public void mouseExited(java.awt.event.MouseEvent e) {}
  public void mousePressed(java.awt.event.MouseEvent e) {}
  public void mouseReleased(java.awt.event.MouseEvent e) {}




//***************************************************************************//
//									     //
//				Polling Thread				     //
//									     //
//***************************************************************************//

  public void run() {
    int i;
    boolean changed;

    while(true) {
      try {
        changed=false;
        i=parport.read();
        if(i!=data_in) {
          data_in=i;
          changed=true;
          System.out.println("Input: "+Integer.toHexString(i&0xFF)+", time "+
                             System.currentTimeMillis());
        }

        i=parport.readStatus();
        if(i!=status) {
          status=i;
          changed=true;
        }
        if(changed) repaint();

      } catch (IOException ioe) {
        System.err.println("ButtonCanvas.257: unexpected io exception");
        ioe.printStackTrace();
        System.exit(1);
      }

      try {
        Thread.sleep(10);
      } catch (InterruptedException ie) {}

    }
  }
}
}
