#ifndef _LINUX_PCS_H
#define _LINUX_PCS_H


#define PPORT_VERSION		"0.91"

#define PPORT_MAJOR		62

#define PPORT_MAX_PORTS		4


#define PPORT_WBYTE		0
#define PPORT_WSTATUS		1
#define PPORT_RBYTE		2
#define PPORT_RSTATUS		3
#define PPORT_SEQUENCE		4
#define PPORT_ACTION		7


#define PPORT_IOCTL_MODEOFF	(PPORT_MAJOR<<8)
#define PPORT_IOCTL_MODEON	(PPORT_MAJOR<<8)+1
#define PPORT_WRITESTATUS	(PPORT_MAJOR<<8)+2
#define PPORT_READSTATUS	(PPORT_MAJOR<<8)+3
#define PPORT_DOSEQUENCE	(PPORT_MAJOR<<8)+4

#define PPORT_IOCTL_CMD		0x7


#define PPORT_PROBE_DELAY	10
#define PPORT_DUMMY		0xAA


#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/uaccess.h>

#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/sched.h>


struct parport {
        unsigned long		base;     /* base address */
        unsigned char		flags;

// the main input buffer
	unsigned char		*indata;
// the size of the input buffer
	int			indatabuffersize;

// the position where to place the next input byte
        int			indatapos;


// buffer for read data (in NON-IOCTL mode)
// put data to head, read from tail
        unsigned char		*outdata;
// size of read data buffer
	int			outdatabuffersize;

// offset to head of read data where new data is added
	int 			outdatahead;

// total length of data in outdata, 
	int			outdatalength;


	char			devname[16];
};


#define PPORT_EXISTS		0x01
#define PPORT_IS_OPEN		0x02	/* means /dev/pport is in use	*/
#define PPORT_IOCTL_MODE	0x04	/* if set status regs are only read
					   with ioctl calls		*/

/* the first time the output buffer overflows, a warning is
 * written to the kernel log and the flag is set. future
 * overflows will then be silent.
 */
#define PPORT_OUT_OVERFLOW	0x08


static long long pport_lseek(struct file *file, loff_t offset, int origin);
static ssize_t pport_read(struct file *file, char *buffer, size_t count, loff_t *ppos);
static ssize_t pport_write(struct file *file, const char *buffer, size_t count, loff_t *ppos);
static int pport_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);
static int pport_open(struct inode *inode, struct file *file);
static int pport_release(struct inode * inode, struct file * file);



#endif
