package org.halfdog.io.parport;

import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;

/** Write to data to the pport device. The device just writes
 *  the data into the registers.
 *  The current version of this driver is in project 7
*/

public class LinuxParPort extends ParPort {

  private String		device;
  private InputStream		in;
  private OutputStream		out;
  private byte			buf[];

  public LinuxParPort(String device) throws IOException {
    this.device=device;
    in=new java.io.FileInputStream(device);
    out=new java.io.FileOutputStream(device);

    buf=new byte[16];
  }



  public byte read() throws IOException {
    int i;

    out.write(PPORT_RDATA);
    while((i=in.read())==-1) Thread.yield();
    return((byte)i);
  }


  public synchronized void write(byte b) throws IOException {
    buf[0]=PPORT_WDATA;
    buf[1]=b;
    out.write(buf, 0, 2);
  }





  public byte readStatus() throws IOException {
    int i;

    out.write(PPORT_RSTATUS);
    while((i=in.read())==-1) Thread.yield();
    return((byte)(i));
  }
  public void writeControl(byte b) throws IOException {
    buf[0]=PPORT_WCTRL;
    buf[1]=(byte)(b);
    out.write(buf, 0, 2);
  }



  public Sequence createSequence() {
    return(new LinuxSequence());
  }



/** Execute a sequence. This feature is used to execute timing
 *  critical commands in a very fast way.
*/
  public byte[] doSequence(Sequence seq) throws IOException {

    byte data[]=((LinuxSequence)seq).getSequence();
    boolean stats[]=((LinuxSequence)seq).getStatusExpects();

    byte buf[]=new byte[stats.length];
    out.write(data);
    
    for(int i=0; i<stats.length; ) {
      int n=in.read();
      if(n>=0) {
        buf[i]=(byte)n;
        i++;
      }
    }

    return(buf);
  }
}
