package org.halfdog.io.parport;

public class LinuxSequence implements Sequence {
  java.util.Vector	seq;

/** The compiled binary representation of this sequence, suitable
 *  to be sent to the parport linux module.
 */
  byte			seq_bytes[];

/** If true, the seq_bytes data is out of data and has to be
 *  recompiled before returning it to the user.
 */
  boolean		changed;

/** List containing one boolean for every read command in the
 *  sequence. If the value is true then the read is a status
 *  read.
 */
  boolean		exp_status[];

  int			reads;
  



  protected LinuxSequence() {
    seq=new java.util.Vector(16,16);
    clear();
  }



  public void clear() {
    seq.clear();
    seq_bytes=null;
    changed=false;
    reads=0;
  }




/** Add a new step to the end of a sequence.
 *  @param wait the number of microseconds to wait after execution
 *  of the command.
 *  @return the number of the step in the sequence
 */
  public int addStep(int command, int value, int wait) {
//    System.err.println("Adding step "+command+", "+value+", "+wait);
    int i=seq.size();
    seq.addElement(new int[]{command, value, wait});
    changed=true;
    return(i);
  }


/** Get the binary sequence data representing this LinuxSequence.
 *  @return data suitable to be sent to the linux parport device
 * driver.
 */
  protected byte[] getSequence() {
    if(changed) genSequence();
    return(seq_bytes);
  }


  protected boolean[] getStatusExpects() {
    if(changed) genSequence();
    return(exp_status);
  }



/** Build the byte oriented sequence data for the stored
 *  commands.
 */
  private void genSequence() {
    if((seq_bytes==null)||(seq_bytes.length!=((seq.size()<<2)+5))) {
      seq_bytes=new byte[(seq.size()<<2)+5];
    }
    reads=0;

    int fillpos=0;
    seq_bytes[fillpos++]=ParPort.PPORT_SEQUENCE;

    int i=seq.size();
    for(int n=0; n<4; n++) {
      seq_bytes[fillpos++]=(byte)(i&0xFF);
      i=(i>>8);
    }

    for(i=0; i<seq.size(); i++) {
      int vals[]=(int[])seq.elementAt(i);
      seq_bytes[fillpos++]=(byte)vals[0];

      switch(vals[0]) {
        case ParPort.PPORT_RDATA:
        case ParPort.PPORT_RSTATUS:
          reads++;
        case ParPort.PPORT_WCTRL:
          seq_bytes[fillpos]=(byte)(vals[1]);
          break;
        case ParPort.PPORT_WDATA:
          seq_bytes[fillpos]=(byte)vals[1];
          break;
        default:
          throw(new IllegalArgumentException("Illegal command "+vals[0]+
                                             " in sequence at position "+i));
      }
      fillpos++;

      seq_bytes[fillpos++]=(byte)(vals[2]);
      seq_bytes[fillpos++]=(byte)((vals[2]>>8)&0xFF);
    }


    exp_status=new boolean[reads];
    int readpos=0;
    for(int n=5; n<seq_bytes.length; n+=4) {
      if((seq_bytes[n]==ParPort.PPORT_RDATA)||
         (seq_bytes[n]==ParPort.PPORT_RSTATUS)) {
        exp_status[readpos++]=(seq_bytes[n]==ParPort.PPORT_RSTATUS);
      }
    }

    changed=false;
  }

}
