package org.halfdog.io.parport;

/** Interface for parallel port access. System specific
 *  implementations of this interface grant access to the
 *  parallel port avoiding ioctls or port write commands. <Br>
 * <B>Warning:</B> Writing data into the control register will
 *  put the data as is into the reg, setting a bit high may
 *  result in a low level on the corresponding output line.
 *  This is because some lines are inverted. You have to xor
 *  the value with CTRL_INVERT to get an electrical high
 *  for each logical 1 bit.
 *  The current version of this driver is in project 7
 */

public abstract class ParPort {

  public final static byte PPORT_WDATA		= 0;
  public final static byte PPORT_WCTRL		= 1;
  public final static byte PPORT_RDATA		= 2;
  public final static byte PPORT_RSTATUS	= 3;
  public final static byte PPORT_SEQUENCE	= 4;

/** Control flags to set the control lines with the
 *  writeStatus method. If a flag is set the corresponding
 *  line is set to 5V, no matter if signal has to be
 *  internally inverted or not.
 */
  public final static int CTRL_STROBE		= 0x01;
  public final static int CTRL_AUTOFEED		= 0x02;
  public final static int CTRL_INIT		= 0x04;
  public final static int CTRL_SELECT		= 0x08;
  public final static int CTRL_DIRECTION	= 0x20; // 0_write 1_read

  public final static int STATUS_ERROR		= 0x08;
  public final static int STATUS_SELECT		= 0x10;
  public final static int STATUS_NOPAPER	= 0x20;
  public final static int STATUS_ACKN		= 0x40;
  public final static int STATUS_BUSY		= 0x80;

  public final static int CTRL_INVERT		= 0xB;
  public final static int STATUS_INVERT		= 0x80;


/** Get a parallel port object matching the current os
 */
  public static ParPort createPort(String dev) throws java.io.IOException {
    return(new LinuxParPort(dev));
  }

  public abstract byte read() throws java.io.IOException;

  public abstract void write(byte b) throws java.io.IOException;


  public abstract byte readStatus() throws java.io.IOException;

  public abstract void writeControl(byte b) throws java.io.IOException;


  public abstract Sequence createSequence();

  public abstract byte[] doSequence(Sequence seq) throws java.io.IOException;

}




